@@ -14,6 +14,7 @@
 #include "../raft.h"
 #include "../raft/recv_install_snapshot.h"
 #include "../utils.h"
+#include "../lib/threadpool.h"
 
 /**
  * =Overview
@@ -93,10 +94,9 @@
  * job. Once the background job is finished, the Follower replies with
  * CPResult() or MVResult().
  *
- * 8. When the iteration has finished the Leader sends
- * InstallShapshot(..., done=true) message to the Follower. It moves the
- * Follower back to NORMAL state and the state machine corresponding to the
- * Follower on the Leader is moved to SNAPSHOT_DONE state.
+ * 8. When the iteration has finished the Leader moves into SNAP_DONE state and
+ * sends InstallShapshot(..., done=true) message to the Follower. When Follower
+ * replies, both state machines move to their FINAL states.
  *
  * 9. The Leader sends AppendEntries() RPC to the Follower and restarts the
  * algorithm from (1). The Leader's state machine is being moved to
@@ -241,8 +241,11 @@
  * |       +-------  CHUNK_APPLIED                            |
  * |       |               V Chunk has been written to disk.  |
  * |       +-------  CHUNK_REPLIED ---------------------------+
- * |    (@ || %)           | CP()/MV() had done=true.
- * |                       V CPResult()/MVResult() sent.
+ * |       |               | CP()/MV() had done=true.
+ * |       |               V CPResult()/MVResult() sent.
+ * |       +---------  SNAP_DONE
+ * |    (@ || %)           | InstallSnapshot(done=true) received,
+ * |                       V and reply sent.
  * |                     FINAL
  * |                       |
  * +-----------------------+
@@ -301,11 +304,11 @@ static const struct sm_conf rpc_sm_conf[RPC_NR] = {
 	},
 	[RPC_ERROR] = {
 		.name    = "error",
-		.allowed = BITS(RPC_INIT),
 		.flags   = SM_FINAL,
 	},
 	[RPC_END] = {
 		.name    = "end",
+		.allowed = BITS(RPC_END),
 		.flags   = SM_FINAL,
 	},
 };
@@ -369,12 +372,6 @@ static const struct sm_conf to_sm_conf[TO_NR] = {
 #define M_TIMEOUT    ((const struct raft_message *) 2)
 #define M_WORK_DONE  ((const struct raft_message *) 1)
 
-static bool is_main_thread(void)
-{
-	// TODO: thread local storage.
-	return true;
-}
-
 static bool work_sm_invariant(const struct sm *sm, int prev_state)
 {
 	(void)sm;
@@ -410,17 +407,23 @@ static bool to_sm_invariant(const struct sm *sm, int prev_state)
 	return true;
 }
 
-static void leader_work_done(struct work *w)
+static void leader_work_done(pool_work_t *w)
 {
-	struct leader *leader = CONTAINER_OF(w, struct leader, work);
-	sm_move(&w->sm, WORK_DONE);
+	struct work *work = CONTAINER_OF(w, struct work, pool_work);
+	struct leader *leader = CONTAINER_OF(work, struct leader, work);
+
+	PRE(!leader->ops->is_pool_thread());
+	sm_move(&work->sm, WORK_DONE);
 	leader_tick(leader, M_WORK_DONE);
 }
 
-static void follower_work_done(struct work *w)
+static void follower_work_done(pool_work_t *w)
 {
-	struct follower *follower = CONTAINER_OF(w, struct follower, work);
-	sm_move(&w->sm, WORK_DONE);
+	struct work *work = CONTAINER_OF(w, struct work, pool_work);
+	struct follower *follower = CONTAINER_OF(work, struct follower, work);
+
+	PRE(!follower->ops->is_pool_thread());
+	sm_move(&work->sm, WORK_DONE);
 	follower_tick(follower, M_WORK_DONE);
 }
 
@@ -429,6 +432,11 @@ static void rpc_to_cb(uv_timer_t *handle)
 	struct timeout *to = CONTAINER_OF(handle, struct timeout, handle);
 	struct rpc *rpc = CONTAINER_OF(to, struct rpc, timeout);
 	struct leader *leader = CONTAINER_OF(rpc, struct leader, rpc);
+
+	PRE(!leader->ops->is_pool_thread());
+	if (sm_state(&to->sm) == TO_CANCELED) {
+		return;
+	}
 	sm_move(&to->sm, TO_EXPIRED);
 	sm_move(&rpc->sm, RPC_TIMEDOUT);
 	leader_tick(leader, M_TIMEOUT);
@@ -438,6 +446,11 @@ static void leader_to_cb(uv_timer_t *handle)
 {
 	struct timeout *to = CONTAINER_OF(handle, struct timeout, handle);
 	struct leader *leader = CONTAINER_OF(to, struct leader, timeout);
+
+	PRE(!leader->ops->is_pool_thread());
+	if (sm_state(&to->sm) == TO_CANCELED) {
+		return;
+	}
 	sm_move(&to->sm, TO_EXPIRED);
 	leader_tick(leader, M_TIMEOUT);
 }
@@ -465,11 +478,11 @@ static void leader_sent_cb(struct sender *s, int rc)
 	struct rpc *rpc = CONTAINER_OF(s, struct rpc, sender);
 	struct leader *leader = CONTAINER_OF(rpc, struct leader, rpc);
 
+	PRE(!leader->ops->is_pool_thread());
 	if (UNLIKELY(rc != 0)) {
 		sm_move(&rpc->sm, RPC_ERROR);
 		return;
 	}
-
 	leader_tick(leader, M_MSG_SENT);
 }
 
@@ -478,32 +491,101 @@ static void follower_sent_cb(struct sender *s, int rc)
 	struct rpc *rpc = CONTAINER_OF(s, struct rpc, sender);
 	struct follower *follower = CONTAINER_OF(rpc, struct follower, rpc);
 
+	PRE(!follower->ops->is_pool_thread());
 	if (UNLIKELY(rc != 0)) {
 		sm_move(&rpc->sm, RPC_ERROR);
 		return;
 	}
-
 	follower_tick(follower, M_MSG_SENT);
 }
 
 static bool is_a_trigger_leader(const struct leader *leader, const struct raft_message *incoming)
 {
-	(void)leader;
-	(void)incoming;
-	return true;
+	int state = sm_state(&leader->sm);
+
+	/* Special cases: */
+	if (incoming == M_WORK_DONE) {
+		return IN(state, LS_HT_WAIT, LS_PAGE_SENT, LS_PERSISTED_SIG_PART,
+				LS_PAGE_READ);
+	} else if (incoming == M_MSG_SENT) {
+		if (sm_state(&leader->rpc.sm) != RPC_FILLED) {
+			return false;
+		}
+		return IN(state, LS_PAGE_READ, LS_SNAP_DONE, LS_F_NEEDS_SNAP,
+				LS_REQ_SIG_LOOP, LS_CHECK_F_HAS_SIGS);
+	} else if (incoming == M_TIMEOUT) {
+		return IN(state, LS_PAGE_READ, LS_SNAP_DONE, LS_F_NEEDS_SNAP,
+				LS_REQ_SIG_LOOP, LS_CHECK_F_HAS_SIGS, LS_WAIT_SIGS);
+	}
+
+	/* From now on, the message pointer is a valid pointer. */
+	PRE(incoming != M_MSG_SENT && incoming != M_TIMEOUT &&
+			incoming != M_WORK_DONE);
+
+	/* Leader has not send the AppendEntries message but reacts on its
+	 * reply. */
+	if (state == LS_F_ONLINE) {
+		// TODO check if raft entry is present, else it is not a trigger.
+		return incoming->type == RAFT_IO_APPEND_ENTRIES_RESULT;
+	}
+
+	/* Leader is waiting for follower reply. */
+	if (sm_state(&leader->rpc.sm) != RPC_SENT) {
+		/* We are not expecting a reply. */
+		return false;
+	};
+	switch (state) {
+	case LS_PAGE_READ:
+		return IN(incoming->type, RAFT_IO_INSTALL_SNAPSHOT_CP_RESULT,
+				RAFT_IO_INSTALL_SNAPSHOT_MV_RESULT);
+	case LS_SNAP_DONE:
+	case LS_F_NEEDS_SNAP:
+		return incoming->type == RAFT_IO_INSTALL_SNAPSHOT_RESULT;
+	case LS_REQ_SIG_LOOP:
+	case LS_CHECK_F_HAS_SIGS:
+		return incoming->type == RAFT_IO_SIGNATURE_RESULT;
+	}
+	return false;
 }
 
 static bool is_a_trigger_follower(const struct follower *follower,
 		const struct raft_message *incoming)
 {
-	switch (sm_state(&follower->sm)) {
-	case FS_SIGS_CALC_LOOP:
-		return incoming != M_WORK_DONE;
-	case FS_SIG_PROCESSED:
-	case FS_CHUNCK_PROCESSED:
-		return incoming == M_WORK_DONE;
+	int state = sm_state(&follower->sm);
+
+	/* Special cases: */
+	if (incoming == M_WORK_DONE) {
+		return IN(state, FS_SIG_PROCESSED, FS_CHUNCK_PROCESSED,
+				FS_SIG_PROCESSED, FS_CHUNCK_PROCESSED, FS_CHUNCK_REPLIED,
+				FS_HT_WAIT);
+	} else if (incoming == M_MSG_SENT) {
+		if (sm_state(&follower->rpc.sm) != RPC_FILLED) {
+			return false;
+		}
+		return IN(state, FS_NORMAL, FS_SIGS_CALC_LOOP, FS_SIG_READ,
+				FS_CHUNCK_APPLIED, FS_SNAP_DONE);
+	} else if (incoming == M_TIMEOUT) {
+		/* No timeouts in follower. */
+		return false;
 	}
-	return true;
+
+	/* From now on, the message pointer is a valid pointer. */
+	PRE(incoming != M_MSG_SENT && incoming != M_TIMEOUT &&
+			incoming != M_WORK_DONE);
+
+	switch (state) {
+	case FS_NORMAL:
+	case FS_SNAP_DONE:
+		return incoming->type == RAFT_IO_INSTALL_SNAPSHOT;
+	case FS_SIGS_CALC_LOOP:
+		return incoming->type == RAFT_IO_SIGNATURE;
+	case FS_SIG_RECEIVING:
+		return incoming->type == RAFT_IO_SIGNATURE;
+	case FS_CHUNCK_RECEIVING:
+		return IN(incoming->type, RAFT_IO_INSTALL_SNAPSHOT_CP,
+				RAFT_IO_INSTALL_SNAPSHOT_MV);
+	}
+	return false;
 }
 
 static bool is_a_duplicate(const void *state,
@@ -559,16 +641,83 @@ static void work_fill_follower(struct follower *follower)
 
 static void rpc_fill_leader(struct leader *leader)
 {
-	rpc_init(&leader->rpc);
-	sm_relate(&leader->sm, &leader->rpc.sm);
-	sm_move(&leader->rpc.sm, RPC_FILLED);
+	struct rpc *rpc = &leader->rpc;
+
+	rpc_init(rpc);
+	sm_relate(&leader->sm, &rpc->sm);
+
+	switch (sm_state(&leader->sm)) {
+	case LS_PAGE_READ:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_INSTALL_SNAPSHOT_CP, // TODO CP OR MV.
+		};
+		break;
+	case LS_SNAP_DONE:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_INSTALL_SNAPSHOT,
+			.install_snapshot = (struct raft_install_snapshot) {
+				.result = RAFT_SNAPSHOT_DONE,
+			},
+		};
+		break;
+	case LS_F_NEEDS_SNAP:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_INSTALL_SNAPSHOT,
+		};
+		break;
+	case LS_REQ_SIG_LOOP:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_SIGNATURE,
+		};
+		break;
+	case LS_CHECK_F_HAS_SIGS:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_SIGNATURE,
+			.signature = (struct raft_signature) {
+				.ask_calculated = true,
+			},
+		};
+		break;
+	}
+
+	sm_move(&rpc->sm, RPC_FILLED);
 }
 
 static void rpc_fill_follower(struct follower *follower)
 {
-	rpc_init(&follower->rpc);
-	sm_relate(&follower->sm, &follower->rpc.sm);
-	sm_move(&follower->rpc.sm, RPC_FILLED);
+	struct rpc *rpc = &follower->rpc;
+
+	rpc_init(rpc);
+	sm_relate(&follower->sm, &rpc->sm);
+
+	switch (sm_state(&follower->sm)) {
+	case FS_SIGS_CALC_LOOP:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_SIGNATURE_RESULT,
+			.signature_result = (struct raft_signature_result) {
+				.calculated = follower->sigs_calculated,
+			},
+		};
+		break;
+	case FS_SIG_READ:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_SIGNATURE_RESULT,
+		};
+		break;
+	case FS_CHUNCK_APPLIED:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_INSTALL_SNAPSHOT_CP_RESULT,
+		};
+		break;
+	case FS_NORMAL:
+	case FS_SNAP_DONE:
+		rpc->message = (struct raft_message) {
+			.type = RAFT_IO_INSTALL_SNAPSHOT_RESULT,
+		};
+		break;
+	}
+
+	sm_move(&rpc->sm, RPC_FILLED);
 }
 
 static int rpc_send(struct rpc *rpc, sender_send_op op, sender_cb_op sent_cb)
@@ -629,10 +778,10 @@ static bool is_an_unexpected_trigger(const struct leader *leader,
 		return false;
 	}
 
-	enum raft_result res = RAFT_RESULT_UNEXPECTED;
+	enum raft_snapshot_result res = RAFT_SNAPSHOT_UNEXPECTED;
 	switch (msg->type) {
-	case RAFT_IO_APPEND_ENTRIES:
-		res = RAFT_RESULT_OK;
+	case RAFT_IO_APPEND_ENTRIES_RESULT:
+		res = RAFT_SNAPSHOT_OK;
 		break;
 	case RAFT_IO_INSTALL_SNAPSHOT:
 		res = msg->install_snapshot.result;
@@ -659,7 +808,7 @@ static bool is_an_unexpected_trigger(const struct leader *leader,
 		res = msg->signature_result.result;
 		break;
 	}
-	return res == RAFT_RESULT_UNEXPECTED;
+	return res == RAFT_SNAPSHOT_UNEXPECTED;
 }
 
 static int follower_next_state(struct sm *sm)
@@ -701,7 +850,7 @@ __attribute__((unused)) void leader_tick(struct leader *leader, const struct raf
 	struct sm *sm = &leader->sm;
 	const struct leader_ops *ops = leader->ops;
 
-	PRE(is_main_thread());
+	PRE(!ops->is_pool_thread());
 
 	if (!is_a_trigger_leader(leader, incoming) ||
 	    is_a_duplicate(leader, incoming))
@@ -778,7 +927,7 @@ __attribute__((unused)) void follower_tick(struct follower *follower, const stru
 	    is_a_duplicate(follower, incoming))
 		return;
 
-	PRE(is_main_thread());
+	PRE(!ops->is_pool_thread());
 
 again:
 	switch (sm_state(&follower->sm)) {
@@ -786,6 +935,7 @@ again:
 	case FS_SIGS_CALC_LOOP:
 	case FS_SIG_READ:
 	case FS_CHUNCK_APPLIED:
+	case FS_SNAP_DONE:
 		follower_rpc_tick(&follower->rpc);
 		if (sm_state(&follower->rpc.sm) == RPC_SENT) {
 			rpc_fini(&follower->rpc);
@@ -800,7 +950,6 @@ again:
 		break;
 	case FS_SIG_PROCESSED:
 	case FS_CHUNCK_PROCESSED:
-	case FS_CHUNCK_REPLIED:
 	case FS_HT_WAIT:
 		sm_move(sm, follower_next_state(sm));
 		goto again;
@@ -815,6 +964,7 @@ again:
 	case FS_SIG_REPLIED:
 	case FS_SIGS_CALC_DONE:
 	case FS_SIGS_CALC_MSG_RECEIVED:
+	case FS_CHUNCK_REPLIED:
 	case FS_FINAL:
 		sm_move(sm, follower_next_state(sm));
 		break;
@@ -922,4 +1072,3 @@ reply:
 	return 0;
 }
 
-#undef tracef
