@@ -518,8 +518,12 @@ static void appendLeaderCb(struct raft_io_append *append, int status)
 				case RAFT_BARRIER: {
 					struct raft_barrier *barrier =
 					    (struct raft_barrier *)req;
-					if (barrier->cb) {
-						barrier->cb(barrier, status);
+					while (barrier != NULL) {
+						struct raft_barrier *next = barrier->next;
+						if (barrier->cb != NULL) {
+							barrier->cb(barrier, status);
+						}
+						barrier = next;
 					}
 					break;
 				}
@@ -1583,8 +1587,13 @@ static void applyBarrier(struct raft *r, const raft_index index)
 		return;
 	}
 	queue_remove(&req->queue);
-	if (req->cb != NULL) {
-		req->cb(req, 0);
+
+	while (req != NULL) {
+		struct raft_barrier *next = req->next;
+		if (req->cb != NULL) {
+			req->cb(req, 0);
+		}
+		req = next;
 	}
 }
 
@@ -1674,6 +1683,50 @@ static void takeSnapshotClose(struct raft *r, struct raft_snapshot *s)
 	r->fsm->snapshot_finalize(r->fsm, &s->bufs, &s->n_bufs);
 }
 
+
+static unsigned dynamicTrailingIndex(struct raft *r, const struct raft_snapshot *snapshot) {
+	struct raft_log *l = r->log;
+	size_t trailing, snapshot_size = 0, num_entries = logNumEntries(l);
+	unsigned i;
+	/** 
+	 * This should never happen as it would mean that the snapshot
+	 * contains no new entry (i.e. either the snapshot is empty
+	 * or equal to the previous one).
+	 */
+	PRE(num_entries > 0);
+
+	for (i = 0; i < snapshot->n_bufs; i++) {
+		snapshot_size += snapshot->bufs[i].len;
+	}
+
+	for (trailing = 0; trailing < num_entries; trailing++) {
+		const struct raft_entry* entry = logEntryAt(l, num_entries - trailing - 1);
+		if (entry->buf.len > snapshot_size) {
+			break;
+		}
+		snapshot_size -= entry->buf.len;
+	}
+
+	if (trailing < r->snapshot.threshold) {
+		return r->snapshot.threshold;
+	} else if (trailing > r->snapshot.trailing) {
+		return r->snapshot.trailing;
+	}
+
+	return (unsigned)trailing;
+}
+
+
+static unsigned trailingSize(struct raft *r, struct raft_snapshot *snapshot) {
+	switch (r->snapshot.trailing_strategy) {
+	case RAFT_TRAILING_STRATEGY_DYNAMIC:
+		return dynamicTrailingIndex(r, snapshot);
+	case RAFT_TRAILING_STRATEGY_STATIC:
+	default:
+		return r->snapshot.trailing;
+	}
+}
+
 static void takeSnapshotCb(struct raft_io_snapshot_put *req, int status)
 {
 	struct raft *r = req->data;
@@ -1702,7 +1755,7 @@ static void takeSnapshotCb(struct raft_io_snapshot_put *req, int status)
 		 * an aborted configuration change. */
 		tracef("failed to backup last committed configuration.");
 	}
-	logSnapshot(r->log, snapshot->index, r->snapshot.trailing);
+	logSnapshot(r->log, snapshot->index, trailingSize(r, snapshot));
 out:
 	takeSnapshotClose(r, snapshot);
 	r->snapshot.pending.term = 0;
@@ -1936,4 +1989,3 @@ inline bool replicationInstallSnapshotBusy(struct raft *r)
 	return r->last_stored == 0 && r->snapshot.put.data != NULL;
 }
 
-#undef tracef
