@@ -37,18 +37,8 @@ static void state_cb(struct raft *r,
 		     unsigned short old_state,
 		     unsigned short new_state)
 {
-	struct dqlite_node *d = r->data;
-	queue *head;
-	struct conn *conn;
-
 	if (old_state == RAFT_LEADER && new_state != RAFT_LEADER) {
 		tracef("node %llu@%s: leadership lost", r->id, r->address);
-		QUEUE_FOREACH(head, &d->conns)
-		{
-			conn = QUEUE_DATA(head, struct conn, queue);
-			gateway__leader_close(&conn->gateway,
-					      RAFT_LEADERSHIPLOST);
-		}
 	}
 }
 
@@ -198,7 +188,6 @@ void dqlite__close(struct dqlite_node *d)
 	// TODO assert rv of uv_loop_close after fixing cleanup logic related to
 	// the TODO above referencing the cleanup logic without running the
 	// node. See https://github.com/canonical/dqlite/issues/504.
-
 	uv_loop_close(&d->loop);
 	raftProxyClose(&d->raft_transport);
 	registry__close(&d->registry);
@@ -367,9 +356,10 @@ int dqlite_node_set_failure_domain(dqlite_node *n, unsigned long long code)
 	return 0;
 }
 
-int dqlite_node_set_snapshot_params(dqlite_node *n,
-				    unsigned snapshot_threshold,
-				    unsigned snapshot_trailing)
+int dqlite_node_set_snapshot_params_v2(dqlite_node *n,
+					unsigned snapshot_threshold,
+					unsigned snapshot_trailing,
+					int      trailing_strategy)
 {
 	if (n->running) {
 		return DQLITE_MISUSE;
@@ -386,11 +376,29 @@ int dqlite_node_set_snapshot_params(dqlite_node *n,
 		return DQLITE_MISUSE;
 	}
 
+	switch (trailing_strategy) {
+	case DQLITE_SNAPSHOT_TRAILING_STATIC:
+		raft_set_snapshot_trailing_strategy(&n->raft, RAFT_TRAILING_STRATEGY_STATIC);
+		break;
+	case DQLITE_SNAPSHOT_TRAILING_DYNAMIC:
+		raft_set_snapshot_trailing_strategy(&n->raft, RAFT_TRAILING_STRATEGY_DYNAMIC);
+		break;
+	default:
+		return DQLITE_MISUSE;
+	}
+
 	raft_set_snapshot_threshold(&n->raft, snapshot_threshold);
 	raft_set_snapshot_trailing(&n->raft, snapshot_trailing);
 	return 0;
 }
 
+int dqlite_node_set_snapshot_params(dqlite_node *n,
+					unsigned snapshot_threshold,
+					unsigned snapshot_trailing)
+{
+	return dqlite_node_set_snapshot_params_v2(n, snapshot_threshold, snapshot_trailing, DQLITE_SNAPSHOT_TRAILING_STATIC);
+}
+
 #define KB(N) (1024 * N)
 int dqlite_node_set_block_size(dqlite_node *n, size_t size)
 {
@@ -417,6 +425,7 @@ int dqlite_node_set_block_size(dqlite_node *n, size_t size)
 	raft_uv_set_block_size(&n->raft_io, size);
 	return 0;
 }
+
 int dqlite_node_enable_disk_mode(dqlite_node *n)
 {
 	int rv;
@@ -425,7 +434,7 @@ int dqlite_node_enable_disk_mode(dqlite_node *n)
 		return DQLITE_MISUSE;
 	}
 
-	rv = dqlite_vfs_enable_disk(&n->vfs);
+	rv = VfsEnableDisk(&n->vfs);
 	if (rv != 0) {
 		return rv;
 	}
@@ -741,6 +750,12 @@ int dqlite_node_enable_role_management(dqlite_node *n)
 	return 0;
 }
 
+int dqlite_node_set_busy_timeout(dqlite_node *n, unsigned msecs)
+{
+	n->config.busy_timeout = msecs;
+	return 0;
+}
+
 int dqlite_node_set_snapshot_compression(dqlite_node *n, bool enabled)
 {
 	return raft_uv_set_snapshot_compression(&n->raft_io, enabled);
@@ -917,6 +932,11 @@ int dqlite_node_stop(dqlite_node *d)
 	void *result;
 	int rv;
 
+	/* Prevents using @d multiple times from different contexts */
+	if (d->lock_fd == -EBADF) {
+	    return EBADF;
+	}
+
 	rv = uv_async_send(&d->stop);
 	assert(rv == 0);
 
@@ -924,6 +944,7 @@ int dqlite_node_stop(dqlite_node *d)
 	assert(rv == 0);
 
 	release_dir(d->lock_fd);
+	d->lock_fd = -EBADF;
 
 	return (int)((uintptr_t)result);
 }
