@@ -70,7 +70,7 @@ struct connection {
 		struct connection *c = &f->connections[i]; \
 		buffer__close(&c->request);                \
 		buffer__close(&c->response);               \
-		gateway__close(&c->gateway);               \
+		gateway__close(&c->gateway, fixture_close_cb);               \
 	}                                                  \
 	TEAR_DOWN_CLUSTER;
 
@@ -86,6 +86,8 @@ static void fixture_handle_cb(struct handle *req,
 	c->schema = schema;
 }
 
+static void fixture_close_cb(struct gateway *g) { (void)g; }
+
 /******************************************************************************
  *
  * Helper macros.
@@ -246,7 +248,7 @@ TEST_CASE(exec, open, NULL)
 	EXEC(f->c1, f->stmt_id1);
 	EXEC(f->c2, f->stmt_id2);
 	WAIT(f->c2);
-	ASSERT_CALLBACK(f->c2, 0, FAILURE);
+	ASSERT_CALLBACK(f->c2, SQLITE_BUSY, FAILURE);
 	ASSERT_FAILURE(f->c2, SQLITE_BUSY, "database is locked");
 	WAIT(f->c1);
 	ASSERT_CALLBACK(f->c1, 0, RESULT);
@@ -272,13 +274,239 @@ TEST_CASE(exec, tx, NULL)
 	EXEC(f->c1, f->stmt_id1);
 	EXEC(f->c2, f->stmt_id2);
 	WAIT(f->c2);
-	ASSERT_CALLBACK(f->c2, 0, FAILURE);
+	ASSERT_CALLBACK(f->c2, SQLITE_BUSY, FAILURE);
 	ASSERT_FAILURE(f->c2, SQLITE_BUSY, "database is locked");
 	WAIT(f->c1);
 	ASSERT_CALLBACK(f->c1, 0, RESULT);
 	return MUNIT_OK;
 }
 
+TEST_CASE(exec, busy_wait_statement, NULL)
+{
+	struct exec_fixture *f = data;
+	(void)params;
+
+	raft_fixture_set_work_duration(&f->cluster, 0, 50);
+	f->servers[0].config.busy_timeout = 100;
+
+	/* Create a test table using connection 0 */
+	PREPARE(f->c1, "CREATE TABLE test (n INT)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+	
+	PREPARE(f->c1, "INSERT INTO test(n) VALUES(1)", &f->stmt_id1);
+	PREPARE(f->c2, "INSERT INTO test(n) VALUES(1)", &f->stmt_id2);
+
+	EXEC(f->c1, f->stmt_id1);
+	EXEC(f->c2, f->stmt_id2);
+	WAIT(f->c2);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c2, SQLITE_OK, RESULT);
+	ASSERT_CALLBACK(f->c1, SQLITE_OK, RESULT);
+	return MUNIT_OK;
+}
+
+TEST_CASE(exec, busy_wait_transaction, NULL)
+{
+	struct exec_fixture *f = data;
+	(void)params;
+
+	raft_fixture_set_work_duration(&f->cluster, 0, 50);
+	f->servers[0].config.busy_timeout = 100;
+
+	/* Create a test table using connection 0 */
+	PREPARE(f->c1, "CREATE TABLE test (n INT)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	PREPARE(f->c1, "BEGIN", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+	
+	/* make sure the write lock is taken */
+	PREPARE(f->c1, "INSERT INTO test(n) VALUES(1)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	/* start another write */
+	PREPARE(f->c2, "INSERT INTO test(n) VALUES(1)", &f->stmt_id2);
+	EXEC(f->c2, f->stmt_id2);
+	
+	PREPARE(f->c1, "COMMIT", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	/* make sure the other write could not progress */
+	munit_assert_false(f->c2->context.invoked);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	/* make sure the other write is correctly dequeued */
+	WAIT(f->c2);
+	ASSERT_CALLBACK(f->c2, 0, RESULT);
+	return MUNIT_OK;
+}
+
+
+TEST_CASE(exec, busy_wait_transaction_dropped, NULL)
+{
+	struct exec_fixture *f = data;
+	(void)params;
+	
+	f->servers[0].config.busy_timeout = 100;
+
+	/* Create a test table using connection 0 */
+	PREPARE(f->c1, "CREATE TABLE test (n INT)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	/* make sure the write lock is taken */
+	PREPARE(f->c1, "BEGIN IMMEDIATE", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+	
+	/* start another write */
+	PREPARE(f->c2, "INSERT INTO test(n) VALUES(1)", &f->stmt_id2);
+	EXEC(f->c2, f->stmt_id2);
+	
+
+	PREPARE(f->c1, "INSERT INTO test(n) VALUES(1)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+	munit_assert_false(f->c2->context.invoked);
+
+	gateway__close(&f->c1->gateway, fixture_close_cb);
+	munit_assert_ptr(f->c1->gateway.leader, ==, NULL);
+
+	/* make sure the other write is correctly dequeued */
+	WAIT(f->c2);
+	ASSERT_CALLBACK(f->c2, 0, RESULT);
+	return MUNIT_OK;
+}
+
+TEST_CASE(exec, busy_wait_timeout, NULL)
+{
+	struct exec_fixture *f = data;
+	(void)params;
+	
+	f->servers[0].config.busy_timeout = 10;
+
+	/* Create a test table using connection 0 */
+	PREPARE(f->c1, "CREATE TABLE test (n INT)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	PREPARE(f->c1, "BEGIN", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+	
+	/* make sure the write lock is taken */
+	PREPARE(f->c1, "INSERT INTO test(n) VALUES(1)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	/* try to write from another connection should fail after some time */
+	PREPARE(f->c2, "INSERT INTO test(n) VALUES(1)", &f->stmt_id2);
+	EXEC(f->c2, f->stmt_id2);
+	WAIT(f->c2);
+	ASSERT_CALLBACK(f->c2, SQLITE_BUSY, FAILURE);
+	
+	/* the original write should still finish correctly */
+	PREPARE(f->c1, "COMMIT", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	return MUNIT_OK;
+}
+
+static int faultyStartTimer(struct raft_io *io,
+			    struct raft_timer *req,
+			    uint64_t timeout,
+			    uint64_t repeat,
+			    raft_timer_cb cb)
+{
+	(void)io;
+	(void)req;
+	(void)timeout;
+	(void)repeat;
+	(void)cb;
+	
+	return RAFT_ERROR;
+}
+
+TEST_CASE(exec, busy_wait_timer_failed, NULL)
+{
+	struct exec_fixture *f = data;
+	(void)params;
+	
+	f->servers[0].config.busy_timeout = 10;
+
+	/* Create a test table using connection 0 */
+	PREPARE(f->c1, "BEGIN IMMEDIATE", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	/* try to write from another connection should fail after some time */
+	CLUSTER_RAFT(0)->io->timer_start = faultyStartTimer;
+	PREPARE(f->c2, "BEGIN IMMEDIATE", &f->stmt_id2);
+	EXEC(f->c2, f->stmt_id2);
+	WAIT(f->c2);
+	ASSERT_CALLBACK(f->c2, SQLITE_IOERR, FAILURE);
+	ASSERT_FAILURE(f->c2, SQLITE_IOERR, "leader exec failed")
+
+	return MUNIT_OK;
+}
+
+
+TEST_CASE(exec, serialization_error, NULL)
+{
+	struct exec_fixture *f = data;
+	(void)params;
+
+	PREPARE(f->c1, "CREATE TABLE test(id)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	/* Create a read transaction */
+	PREPARE(f->c1, "BEGIN", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, RESULT);
+
+	PREPARE(f->c1, "SELECT * FROM test", &f->stmt_id1);
+	QUERY(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, 0, ROWS);
+
+	/* From another connection, create an update, so that
+	 * the transaction above cannot be upgraded anymore
+	 * to a write transaction. */
+	PREPARE(f->c2, "INSERT INTO test(id) VALUES (1)", &f->stmt_id1);
+	EXEC(f->c2, f->stmt_id1);
+	WAIT(f->c2);
+	ASSERT_CALLBACK(f->c2, 0, RESULT);
+
+	/* The original transaction should receive a serialization error*/
+	PREPARE(f->c1, "INSERT INTO test(id) VALUES (2)", &f->stmt_id1);
+	EXEC(f->c1, f->stmt_id1);
+	WAIT(f->c1);
+	ASSERT_CALLBACK(f->c1, SQLITE_BUSY_SNAPSHOT, FAILURE);
+	ASSERT_FAILURE(f->c1, SQLITE_BUSY_SNAPSHOT, "database is locked");
+
+	return MUNIT_OK;
+}
+
 /******************************************************************************
  *
  * Concurrent query requests
